@@ -38,7 +38,7 @@
  *     achieve such a feature it has request sink pads (sink&percnt;u) and
  *     'sometimes' src pads (src&percnt;u).
  *   </para><para>
- *     When requesting a given sinkpad with gst_element_get_request_pad(),
+ *     When requesting a given sinkpad with gst_element_request_pad(),
  *     the associated srcpad for that stream will be created.
  *     Example: requesting sink1 will generate src1.
  *   </para></listitem>
@@ -101,8 +101,6 @@
  *   Both signals are emitted from the context of the streaming thread.
  * </para>
  * </refsect2>
- *
- * Last reviewed on 2008-01-25 (0.10.17)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -146,6 +144,9 @@ struct _GstSingleQueue
   /* segments */
   GstSegment sink_segment;
   GstSegment src_segment;
+  gboolean has_src_segment;     /* preferred over initializing the src_segment to
+                                 * UNDEFINED as this doesn't requires adding ifs
+                                 * in every segment usage */
 
   /* position of src/sink */
   GstClockTime sinktime, srctime;
@@ -198,6 +199,9 @@ static void compute_high_time (GstMultiQueue * mq);
 static void single_queue_overrun_cb (GstDataQueue * dq, GstSingleQueue * sq);
 static void single_queue_underrun_cb (GstDataQueue * dq, GstSingleQueue * sq);
 
+static void update_buffering (GstMultiQueue * mq, GstSingleQueue * sq);
+static void gst_multi_queue_post_buffering (GstMultiQueue * mq);
+
 static void gst_single_queue_flush_queue (GstSingleQueue * sq, gboolean full);
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink_%u",
@@ -268,6 +272,14 @@ enum
   g_mutex_unlock (&q->qlock);                                            \
 } G_STMT_END
 
+#define SET_PERCENT(mq, perc) G_STMT_START {                             \
+  if (perc != mq->percent) {                                             \
+    mq->percent = perc;                                                  \
+    mq->percent_changed = TRUE;                                          \
+    GST_DEBUG_OBJECT (mq, "buffering %d percent", perc);                 \
+  }                                                                      \
+} G_STMT_END
+
 static void gst_multi_queue_finalize (GObject * object);
 static void gst_multi_queue_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -303,7 +315,7 @@ gst_multi_queue_class_init (GstMultiQueueClass * klass)
 
   /**
    * GstMultiQueue::underrun:
-   * @multiqueue: the multqueue instance
+   * @multiqueue: the multiqueue instance
    *
    * This signal is emitted from the streaming thread when there is
    * no data in any of the queues inside the multiqueue instance (underrun).
@@ -371,7 +383,7 @@ gst_multi_queue_class_init (GstMultiQueueClass * klass)
    * GstMultiQueue:use-buffering
    * 
    * Enable the buffering option in multiqueue so that BUFFERING messages are
-   * emited based on low-/high-percent thresholds.
+   * emitted based on low-/high-percent thresholds.
    */
   g_object_class_install_property (gobject_class, PROP_USE_BUFFERING,
       g_param_spec_boolean ("use-buffering", "Use buffering",
@@ -454,6 +466,7 @@ gst_multi_queue_init (GstMultiQueue * mqueue)
   mqueue->high_time = GST_CLOCK_TIME_NONE;
 
   g_mutex_init (&mqueue->qlock);
+  g_mutex_init (&mqueue->buffering_post_lock);
 }
 
 static void
@@ -468,6 +481,7 @@ gst_multi_queue_finalize (GObject * object)
 
   /* free/unref instance data */
   g_mutex_clear (&mqueue->qlock);
+  g_mutex_clear (&mqueue->buffering_post_lock);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -477,6 +491,8 @@ gst_multi_queue_finalize (GObject * object)
     while (tmp) {						\
       GstSingleQueue *q = (GstSingleQueue*)tmp->data;		\
       q->max_size.format = mq->max_size.format;                 \
+      update_buffering (mq, q);                                 \
+      gst_data_queue_limits_changed (q->queue);                 \
       tmp = g_list_next(tmp);					\
     };								\
 } G_STMT_END
@@ -493,6 +509,7 @@ gst_multi_queue_set_property (GObject * object, guint prop_id,
       mq->max_size.bytes = g_value_get_uint (value);
       SET_CHILD_PROPERTY (mq, bytes);
       GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+      gst_multi_queue_post_buffering (mq);
       break;
     case PROP_MAX_SIZE_BUFFERS:
     {
@@ -501,24 +518,34 @@ gst_multi_queue_set_property (GObject * object, guint prop_id,
 
       GST_MULTI_QUEUE_MUTEX_LOCK (mq);
 
+      mq->max_size.visible = new_size;
+
       tmp = mq->queues;
       while (tmp) {
         GstDataQueueSize size;
         GstSingleQueue *q = (GstSingleQueue *) tmp->data;
         gst_data_queue_get_level (q->queue, &size);
 
-        /* do not reduce max size below current level if the single queue has grown because of empty queue */
+        GST_DEBUG_OBJECT (mq, "Queue %d: Requested buffers size: %d,"
+            " current: %d, current max %d", q->id, new_size, size.visible,
+            q->max_size.visible);
+
+        /* do not reduce max size below current level if the single queue
+         * has grown because of empty queue */
         if (new_size == 0) {
           q->max_size.visible = new_size;
+        } else if (q->max_size.visible == 0) {
+          q->max_size.visible = MAX (new_size, size.visible);
         } else if (new_size > size.visible) {
           q->max_size.visible = new_size;
         }
+        update_buffering (mq, q);
+        gst_data_queue_limits_changed (q->queue);
         tmp = g_list_next (tmp);
-      };
-
-      mq->max_size.visible = new_size;
+      }
 
       GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+      gst_multi_queue_post_buffering (mq);
 
       break;
     }
@@ -527,6 +554,7 @@ gst_multi_queue_set_property (GObject * object, guint prop_id,
       mq->max_size.time = g_value_get_uint64 (value);
       SET_CHILD_PROPERTY (mq, time);
       GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+      gst_multi_queue_post_buffering (mq);
       break;
     case PROP_EXTRA_SIZE_BYTES:
       mq->extra_size.bytes = g_value_get_uint (value);
@@ -539,6 +567,31 @@ gst_multi_queue_set_property (GObject * object, guint prop_id,
       break;
     case PROP_USE_BUFFERING:
       mq->use_buffering = g_value_get_boolean (value);
+      if (!mq->use_buffering && mq->buffering) {
+        GST_MULTI_QUEUE_MUTEX_LOCK (mq);
+        mq->buffering = FALSE;
+        GST_DEBUG_OBJECT (mq, "buffering 100 percent");
+        SET_PERCENT (mq, 100);
+        GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+      }
+
+      if (mq->use_buffering) {
+        GList *tmp;
+
+        GST_MULTI_QUEUE_MUTEX_LOCK (mq);
+
+        mq->buffering = TRUE;
+        tmp = mq->queues;
+        while (tmp) {
+          GstSingleQueue *q = (GstSingleQueue *) tmp->data;
+          update_buffering (mq, q);
+          gst_data_queue_limits_changed (q->queue);
+          tmp = g_list_next (tmp);
+        }
+
+        GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+      }
+      gst_multi_queue_post_buffering (mq);
       break;
     case PROP_LOW_PERCENT:
       mq->low_percent = g_value_get_int (value);
@@ -731,6 +784,7 @@ gst_multi_queue_change_state (GstElement * element, GstStateChange transition)
       SET_CHILD_PROPERTY (mqueue, visible);
 
       GST_MULTI_QUEUE_MUTEX_UNLOCK (mqueue);
+      gst_multi_queue_post_buffering (mqueue);
 
       break;
     }
@@ -792,10 +846,12 @@ gst_single_queue_flush (GstMultiQueue * mq, GstSingleQueue * sq, gboolean flush,
     result = gst_pad_pause_task (sq->srcpad);
     sq->sink_tainted = sq->src_tainted = TRUE;
   } else {
-    GST_MULTI_QUEUE_MUTEX_LOCK (mq);
     gst_single_queue_flush_queue (sq, full);
+
+    GST_MULTI_QUEUE_MUTEX_LOCK (mq);
     gst_segment_init (&sq->sink_segment, GST_FORMAT_TIME);
     gst_segment_init (&sq->src_segment, GST_FORMAT_TIME);
+    sq->has_src_segment = FALSE;
     /* All pads start off not-linked for a smooth kick-off */
     sq->srcresult = GST_FLOW_OK;
     sq->pushed = FALSE;
@@ -823,20 +879,16 @@ gst_single_queue_flush (GstMultiQueue * mq, GstSingleQueue * sq, gboolean flush,
   return result;
 }
 
-static void
-update_buffering (GstMultiQueue * mq, GstSingleQueue * sq)
+/* WITH LOCK TAKEN */
+static gint
+get_percentage (GstSingleQueue * sq)
 {
   GstDataQueueSize size;
   gint percent, tmp;
-  gboolean post = FALSE;
-
-  /* nothing to dowhen we are not in buffering mode */
-  if (!mq->use_buffering)
-    return;
 
   gst_data_queue_get_level (sq->queue, &size);
 
-  GST_DEBUG_OBJECT (mq,
+  GST_DEBUG_OBJECT (sq->mqueue,
       "queue %d: visible %u/%u, bytes %u/%u, time %" G_GUINT64_FORMAT "/%"
       G_GUINT64_FORMAT, sq->id, size.visible, sq->max_size.visible,
       size.bytes, sq->max_size.bytes, sq->cur_time, sq->max_size.time);
@@ -856,43 +908,76 @@ update_buffering (GstMultiQueue * mq, GstSingleQueue * sq)
     }
   }
 
+  return percent;
+}
+
+/* WITH LOCK TAKEN */
+static void
+update_buffering (GstMultiQueue * mq, GstSingleQueue * sq)
+{
+  gint percent;
+
+  /* nothing to dowhen we are not in buffering mode */
+  if (!mq->use_buffering)
+    return;
+
+  percent = get_percentage (sq);
+
   if (mq->buffering) {
-    post = TRUE;
     if (percent >= mq->high_percent) {
       mq->buffering = FALSE;
     }
     /* make sure it increases */
     percent = MAX (mq->percent, percent);
 
-    if (percent == mq->percent)
-      /* don't post if nothing changed */
-      post = FALSE;
-    else
-      /* else keep last value we posted */
-      mq->percent = percent;
+    SET_PERCENT (mq, percent);
   } else {
-    if (percent < mq->low_percent) {
+    GList *iter;
+    gboolean is_buffering = TRUE;
+
+    for (iter = mq->queues; iter; iter = g_list_next (iter)) {
+      GstSingleQueue *oq = (GstSingleQueue *) iter->data;
+
+      if (get_percentage (oq) >= 100) {
+        is_buffering = FALSE;
+
+        break;
+      }
+    }
+
+    if (is_buffering && percent < mq->low_percent) {
       mq->buffering = TRUE;
-      mq->percent = percent;
-      post = TRUE;
+      SET_PERCENT (mq, percent);
     }
   }
-  if (post) {
-    GstMessage *message;
+}
+
+static void
+gst_multi_queue_post_buffering (GstMultiQueue * mq)
+{
+  GstMessage *msg = NULL;
+
+  g_mutex_lock (&mq->buffering_post_lock);
+  GST_MULTI_QUEUE_MUTEX_LOCK (mq);
+  if (mq->percent_changed) {
+    gint percent = mq->percent;
+
+    mq->percent_changed = FALSE;
 
-    /* scale to high percent so that it becomes the 100% mark */
     percent = percent * 100 / mq->high_percent;
     /* clip */
     if (percent > 100)
       percent = 100;
 
-    GST_DEBUG_OBJECT (mq, "buffering %d percent", percent);
-    message = gst_message_new_buffering (GST_OBJECT_CAST (mq), percent);
-
-    gst_element_post_message (GST_ELEMENT_CAST (mq), message);
-  } else {
-    GST_DEBUG_OBJECT (mq, "filled %d percent", percent);
+    GST_DEBUG_OBJECT (mq, "Going to post buffering: %d%%", percent);
+    msg = gst_message_new_buffering (GST_OBJECT_CAST (mq), percent);
   }
+  GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+
+  if (msg != NULL)
+    gst_element_post_message (GST_ELEMENT_CAST (mq), msg);
+
+  g_mutex_unlock (&mq->buffering_post_lock);
 }
 
 /* calculate the diff between running time on the sink and src of the queue.
@@ -915,9 +1000,27 @@ update_time_level (GstMultiQueue * mq, GstSingleQueue * sq)
     sink_time = sq->sinktime;
 
   if (sq->src_tainted) {
+    GstSegment *segment;
+    gint64 position;
+
+    if (sq->has_src_segment) {
+      segment = &sq->src_segment;
+      position = sq->src_segment.position;
+    } else {
+      /*
+       * If the src pad had no segment yet, use the sink segment
+       * to avoid signalling overrun if the received sink segment has a
+       * a position > max-size-time while the src pad time would be the default=0
+       *
+       * This can happen when switching pads on chained/adaptive streams and the
+       * new chain has a segment with a much larger position
+       */
+      segment = &sq->sink_segment;
+      position = sq->sink_segment.position;
+    }
+
     src_time = sq->srctime =
-        gst_segment_to_running_time (&sq->src_segment, GST_FORMAT_TIME,
-        sq->src_segment.position);
+        gst_segment_to_running_time (segment, GST_FORMAT_TIME, position);
     /* if we have a time, we become untainted and use the time */
     if (G_UNLIKELY (src_time != GST_CLOCK_TIME_NONE))
       sq->src_tainted = FALSE;
@@ -963,8 +1066,10 @@ apply_segment (GstMultiQueue * mq, GstSingleQueue * sq, GstEvent * event,
 
   if (segment == &sq->sink_segment)
     sq->sink_tainted = TRUE;
-  else
+  else {
+    sq->has_src_segment = TRUE;
     sq->src_tainted = TRUE;
+  }
 
   GST_DEBUG_OBJECT (mq,
       "queue %d, configured SEGMENT %" GST_SEGMENT_FORMAT, sq->id, segment);
@@ -973,6 +1078,7 @@ apply_segment (GstMultiQueue * mq, GstSingleQueue * sq, GstEvent * event,
   update_time_level (mq, sq);
 
   GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+  gst_multi_queue_post_buffering (mq);
 }
 
 /* take a buffer and update segment, updating the time level of the queue. */
@@ -1004,6 +1110,7 @@ apply_buffer (GstMultiQueue * mq, GstSingleQueue * sq, GstClockTime timestamp,
   /* calc diff with other end */
   update_time_level (mq, sq);
   GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+  gst_multi_queue_post_buffering (mq);
 }
 
 static GstClockTime
@@ -1067,7 +1174,7 @@ static GstFlowReturn
 gst_single_queue_push_one (GstMultiQueue * mq, GstSingleQueue * sq,
     GstMiniObject * object)
 {
-  GstFlowReturn result = GST_FLOW_OK;
+  GstFlowReturn result = sq->srcresult;
 
   if (GST_IS_BUFFER (object)) {
     GstBuffer *buffer;
@@ -1382,6 +1489,7 @@ gst_multi_queue_loop (GstPad * pad)
     update_buffering (mq, sq);
 
   GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+  gst_multi_queue_post_buffering (mq);
 
   if (result != GST_FLOW_OK && result != GST_FLOW_NOT_LINKED
       && result != GST_FLOW_EOS)
@@ -1404,7 +1512,19 @@ out_flushing:
     wake_up_next_non_linked (mq);
     sq->last_query = FALSE;
     g_cond_signal (&sq->query_handled);
-    GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+
+    /* Post an error message if we got EOS while downstream
+     * has returned an error flow return. After EOS there
+     * will be no further buffer which could propagate the
+     * error upstream */
+    if (sq->is_eos && sq->srcresult < GST_FLOW_EOS) {
+      GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+      GST_ELEMENT_ERROR (mq, STREAM, FAILED,
+          ("Internal data stream error."),
+          ("streaming stopped, reason %s", gst_flow_get_name (sq->srcresult)));
+    } else {
+      GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+    }
 
     /* upstream needs to see fatal result ASAP to shut things down,
      * but might be stuck in one of our other full queues;
@@ -1604,15 +1724,32 @@ gst_multi_queue_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     goto flushing;
 
   /* mark EOS when we received one, we must do that after putting the
-   * buffer in the queue because EOS marks the buffer as filled. No need to take
-   * a lock, the _check_full happens from this thread only, right before pushing
-   * into dataqueue. */
+   * buffer in the queue because EOS marks the buffer as filled. */
   switch (type) {
     case GST_EVENT_EOS:
+      GST_MULTI_QUEUE_MUTEX_LOCK (mq);
       sq->is_eos = TRUE;
+
+      /* Post an error message if we got EOS while downstream
+       * has returned an error flow return. After EOS there
+       * will be no further buffer which could propagate the
+       * error upstream */
+      if (sq->srcresult < GST_FLOW_EOS) {
+        GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+        GST_ELEMENT_ERROR (mq, STREAM, FAILED,
+            ("Internal data stream error."),
+            ("streaming stopped, reason %s",
+                gst_flow_get_name (sq->srcresult)));
+      } else {
+        GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
+      }
+
       /* EOS affects the buffering state */
+      GST_MULTI_QUEUE_MUTEX_LOCK (mq);
       update_buffering (mq, sq);
+      GST_MULTI_QUEUE_MUTEX_UNLOCK (mq);
       single_queue_overrun_cb (sq->queue, sq);
+      gst_multi_queue_post_buffering (mq);
       break;
     case GST_EVENT_SEGMENT:
       apply_segment (mq, sq, sref, &sq->sink_segment);
@@ -1903,6 +2040,8 @@ single_queue_overrun_cb (GstDataQueue * dq, GstSingleQueue * sq)
   GList *tmp;
   GstDataQueueSize size;
   gboolean filled = TRUE;
+  gboolean all_not_linked = TRUE;
+  gboolean empty_found = FALSE;
 
   gst_data_queue_get_level (sq->queue, &size);
 
@@ -1920,8 +2059,7 @@ single_queue_overrun_cb (GstDataQueue * dq, GstSingleQueue * sq)
     goto done;
   }
 
-  /* if hard limits are not reached then we allow one more buffer in the full
-   * queue, but only if any of the other singelqueues are empty */
+  /* Search for empty or unlinked queues */
   for (tmp = mq->queues; tmp; tmp = g_list_next (tmp)) {
     GstSingleQueue *oq = (GstSingleQueue *) tmp->data;
 
@@ -1933,18 +2071,31 @@ single_queue_overrun_cb (GstDataQueue * dq, GstSingleQueue * sq)
       continue;
     }
 
+    all_not_linked = FALSE;
     GST_LOG_OBJECT (mq, "Checking Queue %d", oq->id);
-
     if (gst_data_queue_is_empty (oq->queue)) {
       GST_LOG_OBJECT (mq, "Queue %d is empty", oq->id);
-      if (IS_FILLED (sq, visible, size.visible)) {
-        sq->max_size.visible = size.visible + 1;
-        GST_DEBUG_OBJECT (mq,
-            "Queue %d is empty, bumping single queue %d max visible to %d",
-            oq->id, sq->id, sq->max_size.visible);
-        filled = FALSE;
-        break;
-      }
+      empty_found = TRUE;
+      break;
+    }
+  }
+
+  if (!mq->queues || !mq->queues->next)
+    all_not_linked = FALSE;
+
+  /* if hard limits are not reached then we allow one more buffer in the full
+   * queue, but only if any of the other singelqueues are empty or all are
+   * not linked */
+  if (all_not_linked || empty_found) {
+    if (all_not_linked) {
+      GST_LOG_OBJECT (mq, "All other queues are not linked");
+    }
+    if (IS_FILLED (sq, visible, size.visible)) {
+      sq->max_size.visible = size.visible + 1;
+      GST_DEBUG_OBJECT (mq,
+          "Bumping single queue %d max visible to %d",
+          sq->id, sq->max_size.visible);
+      filled = FALSE;
     }
   }
 
@@ -2063,6 +2214,11 @@ gst_single_queue_flush_queue (GstSingleQueue * sq, gboolean full)
   gst_data_queue_flush (sq->queue);
   if (was_flushing)
     gst_data_queue_set_flushing (sq->queue, TRUE);
+
+  GST_MULTI_QUEUE_MUTEX_LOCK (sq->mqueue);
+  update_buffering (sq->mqueue, sq);
+  GST_MULTI_QUEUE_MUTEX_UNLOCK (sq->mqueue);
+  gst_multi_queue_post_buffering (sq->mqueue);
 }
 
 static void
